local body_alpha = 0.75
local bees_module_table = {};
local bees_module_limitation = {};
local bees_module_category = {};

function create_drone(r)
	return {
		type = "item",
		name = r.name.."-drone",
		icon_size = 32,
		localised_name = {"item-name.Drone",{"item-name."..r.name}},
		icons = {
			{icon=r.icon},
			{icon="__bees-plus__/graphics/drone-mask.png"},
			{icon="__bees-plus__/graphics/resource_mask.png",tint=r.color}
		},
		flags = {"goes-to-main-inventory"},
		subgroup = r.name.." bees",
		order = "b[3"..r.name.."-drone]",
		stack_size = 200
	}
end

function create_princess(r)
	return {
		type = "item",
		name = r.name.."-princess",
		icon_size = 32,
		localised_name = {"item-name.Princess",{"item-name."..r.name}},
		icons = {
			{icon=r.icon},
			{icon="__bees-plus__/graphics/drone-mask.png"},
			{icon="__bees-plus__/graphics/resource_mask.png",tint=r.color}
		},
		flags = {"goes-to-main-inventory"},
		subgroup = r.name.." bees",
		order = "b[2"..r.name.."-drone]",
		stack_size = 200
	}
end

function create_queen(r)
	return {
		type = "item",
		name = r.name.."-queen",
		icon_size = 32,
		localised_name = {"item-name.Queen",{"item-name."..r.name}},
		icons = {
			{icon=r.icon},
			{icon="__bees-plus__/graphics/drone-mask.png"},
			{icon="__bees-plus__/graphics/resource_mask.png",tint=r.color}
		},
		flags = {"goes-to-main-inventory"},
		subgroup = r.name.." bees",
		order = "b[1"..r.name.."-drone]",
		stack_size = 200
	}
end

function create_comb(r)
	return {
		type = "item",
		name = r.name.."-comb",
		icon_size = 32,
		localised_name = {"item-name.Comb",{"item-name."..r.name}},
		icons = {
			{icon=r.icon},
			{icon="__bees-plus__/graphics/comb-mask.png"},
			{icon="__bees-plus__/graphics/resource_mask.png",tint=r.color}
		},
		flags = {"goes-to-main-inventory"},
		subgroup = r.name.." bees",
		order = "b[4"..r.name.."-comb]",
		stack_size = 200
	}
end

function create_subgroup(r)
	return {
		type = "item-subgroup",
		name = r.name.." bees",
		icon_size = 32,
		group = "Bees",
		order = "a"
	}
end

function breeding_recipe(r)
	local d = {name=r.name.."-drone"};
	local p = {name=r.name.."-princess"};
	local q = {name=r.name.."-queen",icons={{icon=r.icon},{icon="__bees-plus__/graphics/drone-mask.png"},{icon="__bees-plus__/graphics/resource_mask.png",tint=r.color}}};
	local c = {name=r.name.."-comb"};
	table.insert(bees_module_limitation,q.name.." Running");
	return{
		type = "recipe",
		name = d.name.."+"..p.name.." Breeding",
		icon_size = 32,
		localised_name = {"recipe-name.Breeding", {"item-name.Drone",{"item-name."..r.name}}, {"item-name.Princess",{"item-name."..r.name}}, {"item-name.Queen",{"item-name."..r.name}}},
		category = "Beehive-Recipes",
		energy_required = 1,
		subgroup = r.name.." bees",
		order = "b[2"..d.name.."+"..p.name.."="..q.name.."Breeding]",
		enabled = true,
		icons = q.icons,
		ingredients =
		{
			{type="item", name=d.name, amount=1},
			{type="item", name=p.name, amount=1}
		},
		results = {{type="item", name=q.name, amount=1}}
	}
end

function comb_recipe(r)
	local d = {name=r.name.."-drone"};
	local p = {name=r.name.."-princess"};
	local q = {name=r.name.."-queen"};
	local c = {name=r.name.."-comb", icons={{icon=r.icon},{icon="__bees-plus__/graphics/comb-mask.png"},{icon="__bees-plus__/graphics/resource_mask.png",tint=r.color}}};
	table.insert(bees_module_limitation,q.name.." Running");
	return {
		type = "recipe",
		name = q.name.." Running",
		icon_size = 32,
		localised_name = {"recipe-name.Running", {"item-name.Queen",{"item-name."..r.name}}},
		category = "Beehive-Recipes",
		energy_required = 1,
		subgroup = r.name.." bees",
		order = "b[3"..q.name.."Running]",
		enabled = true,
		icons = c.icons,
		ingredients =
		{
			{type="item", name=q.name, amount=1}
		},
		results=
		{
			{type="item", name=d.name, amount=2},
			{type="item", name=p.name, amount=1},
			{type="item", name=c.name, amount=2}
		}		
	}
end

function refine_comb(r)
	local d = {name=r.name.."-drone"};
	local p = {name=r.name.."-princess"};
	local q = {name=r.name.."-queen"};
	local c = {name=r.name.."-comb", icons={{icon=r.icon},{icon="__bees-plus__/graphics/comb-mask.png"},{icon="__bees-plus__/graphics/resource_comb_mask.png",tint=r.color}}};
	return {
		type = "recipe",
		name = c.name.." Refining",
		icon_size = 32,
		localised_name = {"recipe-name.Refining", {"item-name.Comb",{"item-name."..r.name}}},
		category = "crafting",
		energy_required = 1,
		subgroup = r.name.." bees",
		order = "b[4"..r.name.."Comb]",
		enabled = true,
		icons = c.icons,
		ingredients =
		{
			{type="item", name=c.name, amount=1}
		},
		results=
		{
			{type="item", name=r.name, amount=3}
		}		
	}
end

function queen_recipe(r)
	local q = {name=r.name.."-queen",icons={{icon=r.icon},{icon="__bees-plus__/graphics/drone-mask.png"},{icon="__bees-plus__/graphics/resource_mask.png",tint=r.color}}};
	return {
		type = "recipe",
		name = q.name.." Creating",
		icon_size = 32,
		localised_name = {"recipe-name.Creating", {"item-name.Queen",{"item-name."..r.name}}},
		category = "Beehive-Recipes",
		energy_required = 1,
		subgroup = r.name.." bees",
		order = "b[1"..r.name.."Queen]",
		enabled = true,
		icons = q.icons,
		ingredients =
		{
			{type="item", name=r.name, amount=10}
		},
		results=
		{
			{type="item", name=q.name, amount=1}
		}		
	}
end

function scrap_drones(r)
	local d = {name=r.name.."-drone",icons={{icon=r.icon},{icon=r.icon},{icon="__bees-plus__/graphics/drone-mask.png"},{icon="__bees-plus__/graphics/resource_mask.png",tint=r.color}}};
	return {
		type = "recipe",
		name = d.name.." Scraping",
		icon_size = 32,
		localised_name = {"recipe-name.Scraping", {"item-name.Drone",{"item-name."..r.name}}},
		category = "crafting",
		energy_required = 1,
		subgroup = r.name.." bees",
		order = "b[5"..r.name.."Scraping]",
		enabled = true,
		icons = d.icons,
		ingredients =
		{
			{type="item", name=d.name, amount=1}
		},
		results=
		{
			{type="item", name=r.name, amount=1}
		}		
	}
end

function is_fluid(r)
	for _, f in pairs(data.raw.fluid)do
		if(string.find(f.name,r.name))then
			return true;
		end
	end
	return false;
end

function on_list(r)
	if string.find(r.name,"ore") then
		if string.find(r.name,"infinite")then
			return false;
		else
			return true;
		end
	else
		return false;
	end
end

function create_module(r)
	local name = r.name;
	local rname = name.."-frame";
	local image = {{icon = r.icon},{icon = "__bees-plus__/graphics/frame-mask.png"}};
	print("		"..name.." start module")
	return{
		type = "module",
		name = rname,
		icon_size = 32,
		localised_name = {"item-name.Frame",{"item-name."..rname}},
		icons=image,
		flags = {"goes-to-main-inventory"},
		subgroup = "2Frames",
		category = r.category,
		tier = r.tier,
		order = "a["..r.category.."]-a["..rname.."]",
		stack_size = r.stack_size,
		default_request_amount = r.default_request_amount,
		effect = r.effect,
		limitation = bees_module_limitation,
		limitation_message_key = "Frames-work-only-in-beehives"
	}
end

function module_recipe(r)
	local name = r.name;
	local rname = name.."-frame";
	local image = {{icon = r.icon},{icon = "__bees-plus__/graphics/frame-mask.png"}};
	print("		"..name.." start recipe")
	return{
		type = "recipe",
		name = r.name.." Frames",
		icon_size = 32,
		localised_name = {"recipe-name.Frames", {"item-name.Frame",{"item-name."..r.name}}},
		category = "crafting",
		energy_required = 1,
		subgroup = "2Frames",
		order = r.order,
		enabled = true,
		icons = image,
		ingredients =
		{
			{type="item", name=r.name, amount=1}
		},
		results=
		{
			{type="item", name=rname, amount=1}
		}		
	}
end

function load_modules()
	for i, r in pairs(data.raw.module)do
		local name = r.name;
		--if string.find(r.name,"speed") or string.find(r.name,"productivity") or string.find(r.name,"effectivity") then
			print(name.." can be frame")
			table.insert(bees_module_table,r);
		--end
	end
	for _,r in pairs(bees_module_table)do
		data:extend({
			create_module(r);
			module_recipe(r);
		})
	end
end

for i, r in pairs(data.raw.resource) do
	if not is_fluid(r)then
		if on_list(r) then
			local color = {r=math.random(255),g=math.random(255),b=math.random(255),a=math.random(1)};
			table.insert(r,color);
			data:extend({
				create_subgroup(r);
				create_drone(r);
				create_princess(r);
				create_queen(r);
				create_comb(r);
				comb_recipe(r);
				breeding_recipe(r);
				refine_comb(r);
				queen_recipe(r);
				scrap_drones(r);
			})
		end
	end
end

load_modules();